@@ -18,15 +18,16 @@ using namespace Qt::Literals::StringLiterals;
 const QString CONFIG_FILE = u"plasmamobilerc"_s;
 const QString SAVED_CONFIG_GROUP = u"SavedConfig"_s;
 
+// In bin/startplasmamobile, we add `~/.config/plasma-mobile` to XDG_CONFIG_DIRS to overlay our own configs
+const QString MOBILE_KWINRC_FILE = u"plasma-mobile/kwinrc"_s;
+const QString MOBILE_KSMSERVERRC_FILE = u"plasma-mobile/ksmserverrc"_s;
+const QString MOBILE_KDEGLOBALS_FILE = u"plasma-mobile/kdeglobals"_s;
+
 Settings::Settings(QObject *parent)
     : QObject{parent}
     , m_isMobilePlatform{KRuntimePlatform::runtimePlatform().contains(u"phone"_s)}
     , m_mobileConfig{KSharedConfig::openConfig(CONFIG_FILE, KConfig::SimpleConfig)}
-    , m_kwinrcConfig{KSharedConfig::openConfig(u"kwinrc"_s, KConfig::SimpleConfig)}
     , m_appBlacklistConfig{KSharedConfig::openConfig(u"applications-blacklistrc"_s, KConfig::SimpleConfig)}
-    , m_kdeglobalsConfig{KSharedConfig::openConfig(u"kdeglobals"_s, KConfig::SimpleConfig)}
-    , m_ksmServerConfig{KSharedConfig::openConfig(u"ksmserverrc"_s, KConfig::SimpleConfig)}
-    , m_configWatcher{KConfigWatcher::create(m_mobileConfig)}
 {
 }
 
@@ -51,19 +52,20 @@ void Settings::applyConfiguration()
 
 void Settings::loadSavedConfiguration()
 {
-    // kwinrc
-    loadKeys(u"kwinrc"_s, m_kwinrcConfig, getKwinrcSettings(m_mobileConfig));
-    m_kwinrcConfig->sync();
+    // kwinrc (legacy, we only write in the plasma-mobile/kwinrc file now)
+    auto originalKwinrcConfig = KSharedConfig::openConfig(u"kwinrc"_s, KConfig::SimpleConfig);
+    loadKeys(u"kwinrc"_s, originalKwinrcConfig, getKwinrcSettings(m_mobileConfig));
+    originalKwinrcConfig->sync();
     reloadKWinConfig();
 
     // applications-blacklistrc
     loadKeys(u"applications-blacklistrc"_s, m_appBlacklistConfig, APPLICATIONS_BLACKLIST_DEFAULT_SETTINGS);
     m_appBlacklistConfig->sync();
 
-    // kdeglobals
-    loadKeys(u"kdeglobals"_s, m_kdeglobalsConfig, KDEGLOBALS_DEFAULT_SETTINGS);
-    loadKeys(u"kdeglobals"_s, m_kdeglobalsConfig, KDEGLOBALS_SETTINGS);
-    m_kdeglobalsConfig->sync();
+    // kdeglobals (legacy, we only write in the plasma-mobile/kdeglobals file now)
+    auto originalKdeglobalsConfig = KSharedConfig::openConfig(u"kdeglobals"_s, KConfig::SimpleConfig);
+    loadKeys(u"kdeglobals"_s, originalKdeglobalsConfig, KDEGLOBALS_SETTINGS);
+    originalKdeglobalsConfig->sync();
 
     // save our changes
     m_mobileConfig->sync();
@@ -72,32 +74,71 @@ void Settings::loadSavedConfiguration()
 void Settings::applyMobileConfiguration()
 {
     // kwinrc
-    writeKeys(u"kwinrc"_s, m_kwinrcConfig, getKwinrcSettings(m_mobileConfig), false);
-    m_kwinrcConfig->sync();
-    reloadKWinConfig();
+    {
+        auto kwinSettings = getKwinrcSettings(m_mobileConfig);
+        setOptionsImmutable(false, MOBILE_KWINRC_FILE, kwinSettings);
+
+        auto kwinrc = kwinrcConfig();
+        writeKeys(MOBILE_KWINRC_FILE, kwinrc, kwinSettings);
+        kwinrc->sync();
+        reloadKWinConfig();
+
+        setOptionsImmutable(true, MOBILE_KWINRC_FILE, kwinSettings);
+    }
 
     // applications-blacklistrc
-    writeKeys(u"applications-blacklistrc"_s,
-              m_appBlacklistConfig,
-              APPLICATIONS_BLACKLIST_DEFAULT_SETTINGS,
-              true); // only write entries if they are not already defined in the config
-    m_appBlacklistConfig->sync();
+    {
+        writeKeysAndSave(u"applications-blacklistrc"_s,
+                         m_appBlacklistConfig,
+                         APPLICATIONS_BLACKLIST_DEFAULT_SETTINGS,
+                         true); // only write entries if they are not already defined in the config
+        m_appBlacklistConfig->sync();
+    }
 
     // kdeglobals
-    writeKeys(u"kdeglobals"_s, m_kdeglobalsConfig, KDEGLOBALS_DEFAULT_SETTINGS,
-              true); // only write entries if they are not already defined in the config
-    writeKeys(u"kdeglobals"_s, m_kdeglobalsConfig, KDEGLOBALS_SETTINGS, false);
-    m_kdeglobalsConfig->sync();
+    {
+        setOptionsImmutable(false, MOBILE_KDEGLOBALS_FILE, KDEGLOBALS_SETTINGS);
+
+        auto kdeglobals = KSharedConfig::openConfig(MOBILE_KDEGLOBALS_FILE, KConfig::SimpleConfig);
+        writeKeys(MOBILE_KDEGLOBALS_FILE, kdeglobals, KDEGLOBALS_DEFAULT_SETTINGS); // only write, don't make immutable
+        writeKeys(MOBILE_KDEGLOBALS_FILE, kdeglobals, KDEGLOBALS_SETTINGS);
+        kdeglobals->sync();
+
+        setOptionsImmutable(true, MOBILE_KDEGLOBALS_FILE, KDEGLOBALS_SETTINGS);
+    }
 
     // ksmserver
-    writeKeys(u"ksmserverrc"_s, m_ksmServerConfig, KSMSERVER_SETTINGS, false);
-    m_ksmServerConfig->sync();
+    {
+        setOptionsImmutable(false, MOBILE_KSMSERVERRC_FILE, KSMSERVER_SETTINGS);
+
+        auto ksmserver = KSharedConfig::openConfig(MOBILE_KSMSERVERRC_FILE, KConfig::SimpleConfig);
+        writeKeys(MOBILE_KSMSERVERRC_FILE, ksmserver, KSMSERVER_SETTINGS);
+        ksmserver->sync();
+
+        setOptionsImmutable(true, MOBILE_KSMSERVERRC_FILE, KSMSERVER_SETTINGS);
+    }
 
     // save our changes
     m_mobileConfig->sync();
 }
 
-void Settings::writeKeys(const QString &fileName, KSharedConfig::Ptr &config, const QMap<QString, QMap<QString, QVariant>> &settings, bool overwriteOnlyIfEmpty)
+void Settings::writeKeys(const QString &fileName, KSharedConfig::Ptr &config, const QMap<QString, QMap<QString, QVariant>> &settings)
+{
+    const auto groupNames = settings.keys();
+    for (const auto &groupName : groupNames) {
+        auto group = KConfigGroup{config, groupName};
+
+        const auto keys = settings[groupName].keys();
+        for (const auto &key : keys) {
+            group.writeEntry(key, settings[groupName][key], KConfigGroup::Notify);
+        }
+    }
+}
+
+void Settings::writeKeysAndSave(const QString &fileName,
+                                KSharedConfig::Ptr &config,
+                                const QMap<QString, QMap<QString, QVariant>> &settings,
+                                bool overwriteOnlyIfEmpty)
 {
     const auto groupNames = settings.keys();
     for (const auto &groupName : groupNames) {
@@ -174,6 +215,11 @@ const QString Settings::loadSavedConfigSetting(KSharedConfig::Ptr &config, const
     return value;
 }
 
+KSharedConfig::Ptr Settings::kwinrcConfig() const
+{
+    return KSharedConfig::openConfig(MOBILE_KWINRC_FILE, KConfig::SimpleConfig);
+}
+
 void Settings::reloadKWinConfig()
 {
     // Reload config
@@ -182,7 +228,7 @@ void Settings::reloadKWinConfig()
 
     // Effects need to manually be loaded/unloaded in a live KWin session.
 
-    KConfigGroup pluginsGroup{m_kwinrcConfig, QStringLiteral("Plugins")};
+    KConfigGroup pluginsGroup{kwinrcConfig(), QStringLiteral("Plugins")};
 
     for (const auto &effect : KWIN_EFFECTS) {
         // Read from the config whether the effect is enabled (settings are suffixed with "Enabled", ex. blurEnabled)
@@ -209,4 +255,8 @@ void Settings::reloadKWinConfig()
     // Call "start" to load enabled KWin scripts.
     QDBusMessage message = QDBusMessage::createMethodCall(u"org.kde.KWin"_s, u"/Scripting"_s, u"org.kde.kwin.Scripting"_s, u"start"_s);
     QDBusConnection::sessionBus().send(message);
+
+    // Call reconfigure
+    QDBusMessage reconfigureMessage = QDBusMessage::createSignal("/KWin", "org.kde.KWin", "reconfigure");
+    QDBusConnection::sessionBus().send(reconfigureMessage);
 }
